#include <avr/io.h>
#include <avr/interrupt.h>




#define F_CPU				8000000	//frecv ceas
#define PRESCALER			64 //prescaler



#define BASE_FREQUENCY		(F_CPU/PRESCALER)	

#define TUNING_FORK_A		440.0
#define NOTE_DIFF			1.05946309436


#define E_STRING	164.81
#define A_STRING	220.00
#define D_STRING	293.66
#define G_STRING	391.99
#define B_STRING	493.88				
#define EH_STRING	659.26


unsigned int Center_Count[] =
{
	BASE_FREQUENCY/EH_STRING,			// High E
	BASE_FREQUENCY/B_STRING,			// B
	BASE_FREQUENCY/G_STRING,			// G
	BASE_FREQUENCY/D_STRING,			// D
	BASE_FREQUENCY/A_STRING,			// A
	BASE_FREQUENCY/E_STRING,			// Low E
};

unsigned int Transition_Count[] =
{
	BASE_FREQUENCY/(B_STRING+(EH_STRING-B_STRING)/2),	// E to B
	BASE_FREQUENCY/(G_STRING+(B_STRING-G_STRING)/2),		// B to G
	BASE_FREQUENCY/(D_STRING+(G_STRING-D_STRING)/2),		// G to D
	BASE_FREQUENCY/(A_STRING+(D_STRING-A_STRING)/2),		// D to A
	BASE_FREQUENCY/(E_STRING+(A_STRING-E_STRING)/2),		// A to E
};




volatile unsigned char count_hi;	// overflow accumulator

SIGNAL(SIG_OVERFLOW0)	
{
	count_hi++;
						// increment overflow count
}


void Initialise (void)
{
	DDRD = 0xFF;
	DDRB = 0x00;
	//prescaler
	TCCR0B = TCCR0B | 0x03;
	//activez intreruperi
	TIMSK0 = _BV(TOIE0);
	sei();
}

int SampleLoop(int count, int count_hi)
{
		unsigned int i;
			for (i=0;i<32;i++)
			{

			while ((PINB | 0xFE) == 0xFF)		
				if (count_hi > 60)	
					break;			

			while ((PINB | 0xFE) == 0xFE)	
				if (count_hi > 60)
					break;	

			count += (count_hi << 8) + TCNT0; 
			TCNT0 = 0;							 

			if (count_hi > 60)
				break;

			count_hi = 0;
			}
			
			return count;
}

void DetectRange(int count, int count_hi)
{
	int i = 0;
	PORTD = 0x00;
		if (count_hi <= 80)
		{

			count = count >> 5;
	
			
			for (i=0;i<sizeof(Transition_Count)/sizeof(Transition_Count[0]);i++)
			{
				if (count < Transition_Count[i])
					break;
			}				
			
			
			if (count-1 <= Center_Count[i])			// if count <= this string count
				PORTD = 0x80;						// light "Too High" LED
	
			if (count+1 >= Center_Count[i])			// if count >= this string count
				PORTD = 0x40;					// light "Too Low" LED
		}
}

int main(void) 
{
	unsigned int count;
	Initialise();
	while (1)
	{
		count = 0;
		TCNT0 = 0;
		 
		count_hi = 0;
		count = SampleLoop(count, count_hi);
		DetectRange(count, count_hi);		
			
	}
}



 
